"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GptWidgetDecoView = exports.cnGptPopup = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const react_use_1 = require("react-use");
const classname_1 = require("../../../../classname");
const behavior_1 = require("../../../behavior");
const GptDialog_1 = require("../GptDialog/GptDialog");
const constants_1 = require("../constants");
const plugin_1 = require("../plugin");
exports.cnGptPopup = (0, classname_1.cn)('gpt-popup');
class GptWidgetDecoView {
    constructor(view, params) {
        this._decoElem = null;
        this._onConfirmCancel = () => {
            this._confirmOpen = false;
            this._renderer.rerender();
        };
        this._onConfirmOk = () => {
            this._onClose();
        };
        this._onGptAnswerUpdate = (answer) => {
            var _a, _b;
            (_b = (_a = this._params).onUpdate) === null || _b === void 0 ? void 0 : _b.call(_a, answer);
        };
        this._onSubmit = (answer) => {
            var _a, _b;
            const deco = this._getCurrentDecoration();
            if (!deco)
                return;
            const { from, to } = deco;
            const answerNode = this._parser.parse(answer);
            const tr = this._view.state.tr;
            const meta = { action: 'hide' };
            tr.setMeta(plugin_1.pluginKey, meta);
            tr.replace(from, to, new prosemirror_model_1.Slice(answerNode.content, 1, 1));
            tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, tr.mapping.map(to)));
            this._view.dispatch(tr);
            setTimeout(() => {
                this._view.focus();
            }, 0);
            (_b = (_a = this._params).onApplyResult) === null || _b === void 0 ? void 0 : _b.call(_a, answer);
            this._resetState();
        };
        this._onClose = () => {
            var _a, _b;
            const deco = this._getCurrentDecoration();
            if (!deco) {
                return;
            }
            const tr = this._view.state.tr;
            const meta = { action: 'hide' };
            tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, deco.from, deco.to));
            tr.setMeta(plugin_1.pluginKey, meta);
            this._view.dispatch(tr);
            setTimeout(() => {
                this._view.focus();
            }, 0);
            (_b = (_a = this._params).onClose) === null || _b === void 0 ? void 0 : _b.call(_a);
            this._resetState();
        };
        this._view = view;
        this._params = params;
        this._serializer = params.serializer;
        this._parser = params.parser;
        this._confirmOpen = false;
        this._renderer = (0, behavior_1.getReactRendererFromState)(view.state).createItem('gpt-widget-view', this._render.bind(this));
    }
    update(view, prevState) {
        var _a;
        const { state, dispatch } = view;
        if (this._decoElem && !state.selection.eq(prevState.selection)) {
            const transaction = state.tr;
            const meta = { action: 'hide' };
            dispatch(transaction.setMeta(plugin_1.pluginKey, meta));
            this._view.focus();
            return;
        }
        const decoElements = Array.from(view.dom.querySelectorAll(`.${constants_1.WIDGET_DECO_CLASS_NAME}`));
        this._decoElem = (_a = decoElements.at(-1)) !== null && _a !== void 0 ? _a : null;
        this._renderer.rerender();
    }
    destroy() {
        this._resetState();
        this._renderer.remove();
    }
    _resetState() {
        this._decoElem = null;
        this._confirmOpen = false;
    }
    _render() {
        if (!this._decoElem)
            return null;
        const markup = this._getContentOfDecoration();
        if (markup === undefined) {
            return null;
        }
        return (react_1.default.createElement(Widget, { markup: markup, anchorRef: { current: this._decoElem }, answerRender: this._params.answerRender, promptPresets: this._params.promptPresets, disablePromptPresets: this._params.disablePromptPresets, customPromptPlaceholder: this._params.customPromptPlaceholder, disabledPromptPlaceholder: this._params.disabledPromptPlaceholder, gptAlertProps: this._params.gptAlertProps, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onApplyResult: this._onSubmit, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onCustomPromptApply: this._params.onCustomPromptApply, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onPromptPresetClick: this._params.onPromptPresetClick, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onTryAgain: this._params.onTryAgain, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onLike: this._params.onLike, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onDislike: this._params.onDislike, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onClose: this._onClose, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onUpdate: this._onGptAnswerUpdate, 
            /* eslint-disable-next-line react/jsx-handler-names */
            confirmOpen: this._confirmOpen, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onConfirmOk: this._onConfirmOk, 
            /* eslint-disable-next-line react/jsx-handler-names */
            onConfirmCancel: this._onConfirmCancel, container: this._params.gptPopupContainer }));
    }
    _getContentOfDecoration() {
        const deco = this._getCurrentDecoration();
        if (!deco)
            return undefined;
        const { from, to } = deco;
        try {
            const fragment = this._view.state.doc.slice(from, to, true).content;
            const yfmMarkup = this._serializer.serialize(fragment);
            return yfmMarkup;
        }
        catch (error) {
            console.error(error);
            return this._view.state.doc.textBetween(from, to, '\n', '');
        }
    }
    _getCurrentDecoration() {
        var _a;
        return (_a = this._getPluginState()) === null || _a === void 0 ? void 0 : _a.find(undefined, undefined, 
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-return
        (spec) => spec[constants_1.WIDGET_DECO_SPEC_FLAG])[0];
    }
    _getPluginState() {
        return plugin_1.pluginKey.getState(this._view.state);
    }
}
exports.GptWidgetDecoView = GptWidgetDecoView;
function Widget({ markup, anchorRef, answerRender, promptPresets, disablePromptPresets, customPromptPlaceholder, disabledPromptPlaceholder, onCustomPromptApply, onApplyResult, onPromptPresetClick, onTryAgain, onLike, onDislike, onClose, onUpdate, container, gptAlertProps, }) {
    (0, react_use_1.useMount)(() => {
        if ((anchorRef === null || anchorRef === void 0 ? void 0 : anchorRef.current) && 'scrollIntoView' in anchorRef.current) {
            anchorRef.current.scrollIntoView({
                behavior: 'smooth',
                block: 'center',
            });
        }
    });
    const handleUpdate = (0, react_1.useCallback)((result) => {
        onUpdate === null || onUpdate === void 0 ? void 0 : onUpdate(result);
    }, [onUpdate]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(uikit_1.Popup, { className: (0, exports.cnGptPopup)(), contentClassName: (0, exports.cnGptPopup)('content'), open: true, anchorRef: anchorRef, placement: constants_1.gptPopupPlacement, onOutsideClick: onClose, focusTrap: true, autoFocus: !onCustomPromptApply && true, strategy: "absolute", container: container, onEscapeKeyDown: onClose },
            react_1.default.createElement(GptDialog_1.GptDialog, { markup: markup, answerRender: answerRender, promptPresets: promptPresets, disablePromptPresets: disablePromptPresets, customPromptPlaceholder: customPromptPlaceholder, disabledPromptPlaceholder: disabledPromptPlaceholder, onApplyResult: onApplyResult, onCustomPromptApply: onCustomPromptApply, onPromptPresetClick: onPromptPresetClick, onTryAgain: onTryAgain, onLike: onLike, onDislike: onDislike, onClose: onClose, onUpdate: handleUpdate, gptAlertProps: gptAlertProps }))));
}
