"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.render = exports.CommandMenuComponent = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const components_1 = require("@gravity-ui/components");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../../../classname");
const suggest_1 = require("../../../i18n/suggest");
const lodash_1 = require("../../../lodash");
const ErrorBoundary_1 = require("../../../react-utils/ErrorBoundary");
const b = (0, classname_1.cn)('command-menu');
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
const ITEM_HEIGHT = 28; // px
const VISIBLE_ITEMS_COUNT = 10;
const MAX_LIST_HEIGHT = ITEM_HEIGHT * VISIBLE_ITEMS_COUNT; // px
function calcListHeight(itemsCount) {
    if (itemsCount <= 0)
        return undefined;
    return Math.min(MAX_LIST_HEIGHT, itemsCount * ITEM_HEIGHT);
}
const CommandMenuComponent = ({ anchor, currentIndex, items, onClick, onEnterKeyDown, onEscapeKeyDown, onOutsideClick, }) => {
    if (!anchor)
        return null;
    return (react_1.default.createElement(uikit_1.Popup, { open: true, anchorRef: { current: anchor }, placement: placement, onEnterKeyDown: onEnterKeyDown, onEscapeKeyDown: onEscapeKeyDown, onOutsideClick: onOutsideClick },
        react_1.default.createElement("div", { className: b() },
            react_1.default.createElement(uikit_1.List, { virtualized: true, items: items, sortable: false, filterable: false, emptyPlaceholder: (0, suggest_1.i18n)('empty-msg'), itemHeight: ITEM_HEIGHT, itemsHeight: calcListHeight(items.length), renderItem: renderItem, deactivateOnLeave: false, activeItemIndex: currentIndex, onItemClick: (_item, index) => onClick(index), className: b('list'), itemClassName: b('list-item') }))));
};
exports.CommandMenuComponent = CommandMenuComponent;
function renderItem({ id, title, icon, hotkey, hint }) {
    const titleText = (0, lodash_1.isFunction)(title) ? title() : title;
    const hintText = (0, lodash_1.isFunction)(hint) ? hint() : hint;
    return (react_1.default.createElement("div", { key: id, className: b('item', { id }) },
        react_1.default.createElement(uikit_1.Icon, { data: icon.data, size: 20, className: b('item-icon') }),
        react_1.default.createElement("div", { className: b('item-body') },
            react_1.default.createElement("span", { className: b('item-title') }, titleText),
            react_1.default.createElement("div", { className: b('item-extra') },
                hotkey && react_1.default.createElement(uikit_1.Hotkey, { value: hotkey, className: b('item-hotkey') }),
                hintText && react_1.default.createElement(components_1.HelpPopover, { className: b('item- hint'), content: hintText })))));
}
function render(props) {
    return (react_1.default.createElement(ErrorBoundary_1.ErrorLoggerBoundary, null,
        react_1.default.createElement(exports.CommandMenuComponent, Object.assign({}, props))));
}
exports.render = render;
