"use strict";
var _TooltipView_isTooltipOpen;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TooltipView = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const lodash_1 = require("../../../lodash");
const logger_1 = require("../../../logger");
const ErrorBoundary_1 = require("../../../react-utils/ErrorBoundary");
const toolbar_1 = require("../../../toolbar");
const ReactRenderer_1 = require("../ReactRenderer");
const SelectionTooltip = (_a) => {
    var { show, poppupProps } = _a, toolbarProps = tslib_1.__rest(_a, ["show", "poppupProps"]);
    if (!show)
        return null;
    return (react_1.default.createElement(uikit_1.Popup, Object.assign({ open: true }, poppupProps, { style: { padding: '4px 8px' } }),
        react_1.default.createElement(toolbar_1.Toolbar, Object.assign({}, toolbarProps))));
};
class TooltipView {
    constructor(actions, menuConfig) {
        _TooltipView_isTooltipOpen.set(this, false);
        this.baseProps = { show: false, poppupProps: {} };
        this._tooltipRenderItem = null;
        this.actions = actions;
        this.menuConfig = menuConfig;
    }
    get isTooltipOpen() {
        return tslib_1.__classPrivateFieldGet(this, _TooltipView_isTooltipOpen, "f");
    }
    show(view, popupProps) {
        this.view = view;
        tslib_1.__classPrivateFieldSet(this, _TooltipView_isTooltipOpen, true, "f");
        this.baseProps = {
            show: true,
            poppupProps: Object.assign(Object.assign({}, popupProps), this.calcPosition(view)),
        };
        this.renderPopup();
    }
    hide(view) {
        this.view = view;
        tslib_1.__classPrivateFieldSet(this, _TooltipView_isTooltipOpen, false, "f");
        this.baseProps = { show: false, poppupProps: {} };
        this.renderPopup();
    }
    destroy() {
        var _a;
        (_a = this._tooltipRenderItem) === null || _a === void 0 ? void 0 : _a.remove();
        this._tooltipRenderItem = null;
    }
    getSelectionTooltipProps() {
        return Object.assign(Object.assign({}, this.baseProps), { focus: () => this.view.focus(), data: this.getFilteredConfig(), editor: this.actions, onClick: (id) => logger_1.logger.action({ mode: 'wysiwyg', source: 'context-menu', action: id }) });
    }
    getFilteredConfig() {
        return this.baseProps.show
            ? this.menuConfig
                .map((groupData) => groupData.filter((item) => {
                const { condition } = item;
                if (condition === 'enabled') {
                    return item.isEnable(this.actions);
                }
                if ((0, lodash_1.isFunction)(condition)) {
                    return condition(this.view.state);
                }
                return true;
            }))
                .filter((groupData) => Boolean(groupData.length))
            : [];
    }
    renderPopup() {
        this.tooltipRenderItem.rerender();
    }
    get tooltipRenderItem() {
        if (!this._tooltipRenderItem) {
            const reactRenderer = (0, ReactRenderer_1.getReactRendererFromState)(this.view.state);
            this._tooltipRenderItem = reactRenderer.createItem('selection_context', () => (react_1.default.createElement(ErrorBoundary_1.ErrorLoggerBoundary, null,
                react_1.default.createElement(SelectionTooltip, Object.assign({}, this.getSelectionTooltipProps())))));
        }
        return this._tooltipRenderItem;
    }
    calcPosition(view) {
        const viewDom = view.dom;
        // The box in which the tooltip is positioned, to use as base
        const viewBox = viewDom.getBoundingClientRect();
        const viewWidth = viewBox.right - viewBox.left;
        const viewHalfWidth = viewWidth / 2;
        // These are in screen coordinates
        const start = view.coordsAtPos(view.state.selection.from);
        const end = view.coordsAtPos(view.state.selection.to);
        // Find a center-ish x position from the selection endpoints (when
        // crossing lines, end may be more to the left)
        const left = Math.max((start.left + end.left) / 2, start.left + 3);
        const leftOffset = left - (viewBox.left + viewHalfWidth);
        const bottomOffset = -(viewBox.bottom - end.bottom) + this.popupTextOffset;
        return {
            placement: 'bottom',
            anchorRef: { current: viewDom },
            offset: [leftOffset, bottomOffset],
        };
    }
    get popupTextOffset() {
        return 4; // 4px offset from text selection
    }
}
exports.TooltipView = TooltipView;
_TooltipView_isTooltipOpen = new WeakMap();
