"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Color = exports.colorType = exports.colorMarkName = exports.Colors = exports.colorClassName = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const marks_1 = require("../../../utils/marks");
const ColorSpecs_1 = require("./ColorSpecs");
const const_1 = require("./const");
const utils_1 = require("./utils");
var const_2 = require("./const");
Object.defineProperty(exports, "colorClassName", { enumerable: true, get: function () { return const_2.colorClassName; } });
Object.defineProperty(exports, "Colors", { enumerable: true, get: function () { return const_2.Colors; } });
var ColorSpecs_2 = require("./ColorSpecs");
Object.defineProperty(exports, "colorMarkName", { enumerable: true, get: function () { return ColorSpecs_2.colorMarkName; } });
Object.defineProperty(exports, "colorType", { enumerable: true, get: function () { return ColorSpecs_2.colorType; } });
const Color = (builder) => {
    builder.use(ColorSpecs_1.ColorSpecs, {
        validateClassNameColorName: utils_1.validateClassNameColorName,
        parseStyleColorValue: utils_1.parseStyleColorValue,
    });
    builder.addAction(const_1.colorAction, ({ schema }) => {
        const type = (0, ColorSpecs_1.colorType)(schema);
        return {
            isActive: (state) => Boolean((0, marks_1.isMarkActive)(state, type)),
            isEnable: (0, prosemirror_commands_1.toggleMark)(type),
            run: (state, dispatch, _view, attrs) => {
                const params = attrs;
                const hasMark = (0, marks_1.isMarkActive)(state, type);
                if (!params || !params[const_1.colorMarkName]) {
                    if (!hasMark)
                        return true;
                    // remove mark
                    return (0, prosemirror_commands_1.toggleMark)(type, params)(state, dispatch);
                }
                if (hasMark) {
                    // remove old mark, then add new with new color
                    return (0, utils_1.chainAND)((0, prosemirror_commands_1.toggleMark)(type), (0, prosemirror_commands_1.toggleMark)(type, params))(state, dispatch);
                }
                // add mark
                return (0, prosemirror_commands_1.toggleMark)(type, params)(state, dispatch);
            },
            meta(state) {
                var _a;
                return (_a = type.isInSet(state.selection.$to.marks())) === null || _a === void 0 ? void 0 : _a.attrs[const_1.colorMarkName];
            },
        };
    });
};
exports.Color = Color;
