"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageForm = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const is_number_1 = tslib_1.__importDefault(require("is-number"));
const classname_1 = require("../../../../../../classname");
const base_1 = tslib_1.__importDefault(require("../../../../../../forms/base"));
const components_1 = require("../../../../../../forms/components");
const forms_1 = require("../../../../../../i18n/forms");
const useAutoFocus_1 = require("../../../../../../react-utils/useAutoFocus");
const handlers_1 = require("../../../../../../utils/handlers");
const markdown_1 = require("../../../../../markdown");
const specs_1 = require("../../../../../specs");
const b = (0, classname_1.cn)('image-tooltip-form');
const ImageForm = ({ node, updateAttributes, view, unsetEdit, dom }) => {
    const { attrs, marks } = node;
    const link = marks.find((m) => m.type.name === (0, markdown_1.linkType)(view.state.schema).name);
    const [name, setName] = react_1.default.useState(attrs[specs_1.ImgSizeAttr.Title] || '');
    const [alt, setAlt] = react_1.default.useState(attrs[specs_1.ImgSizeAttr.Alt] || '');
    const [width, setWidth] = react_1.default.useState(attrs[specs_1.ImgSizeAttr.Width] || '');
    const [height, setHeight] = react_1.default.useState(attrs[specs_1.ImgSizeAttr.Height] || '');
    const [linkHref, setLinkHref] = react_1.default.useState((link === null || link === void 0 ? void 0 : link.attrs.href) || '');
    const handleSubmit = () => {
        updateAttributes({
            title: name.trim(),
            alt: alt.trim(),
            width: (0, is_number_1.default)(width) && Number(width) >= 0 ? String(width) : '',
            height: (0, is_number_1.default)(height) && Number(height) >= 0 ? String(height) : '',
        }, link
            ? [
                (0, markdown_1.linkType)(view.state.schema).create(Object.assign(Object.assign({}, link.attrs), { [markdown_1.LinkAttr.Href]: linkHref })),
            ]
            : []);
        unsetEdit();
    };
    const linkRef = react_1.default.useRef(null);
    const imageNameRef = react_1.default.useRef(null);
    (0, useAutoFocus_1.useAutoFocus)(link ? linkRef : imageNameRef);
    const inputEnterKeyHandler = (0, handlers_1.enterKeyHandler)(handleSubmit);
    return (react_1.default.createElement(uikit_1.Popup, { open: true, anchorRef: dom, placement: ['bottom-start', 'top-start', 'bottom-end', 'top-end'], onOutsideClick: unsetEdit },
        react_1.default.createElement(base_1.default.Form, { className: b() },
            react_1.default.createElement(base_1.default.Layout, null,
                link && (react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('image_link_href'), help: (0, forms_1.i18n)('image_link_href_help'), control: react_1.default.createElement(uikit_1.TextInput, { controlRef: linkRef, size: "s", view: "normal", value: linkHref, onUpdate: setLinkHref, className: b('input', { type: 'link' }), onKeyPress: inputEnterKeyHandler }) })),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('image_name'), control: react_1.default.createElement(uikit_1.TextInput, { controlRef: imageNameRef, size: "s", view: "normal", value: name, onUpdate: setName, className: b('input', { type: 'name' }), onKeyPress: inputEnterKeyHandler }) }),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('image_alt'), help: (0, forms_1.i18n)('image_alt_help'), control: react_1.default.createElement(uikit_1.TextInput, { size: "s", view: "normal", value: alt, onUpdate: setAlt, className: b('input', { type: 'alt' }), onKeyPress: inputEnterKeyHandler }) }),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('common_sizes'), control: react_1.default.createElement("div", { className: b('size-controls') },
                        react_1.default.createElement(components_1.NumberInput, { min: 0, size: "s", view: "normal", value: width, onUpdate: setWidth, placeholder: (0, forms_1.i18n)('image_size_width'), className: b('input', { type: 'width' }), onKeyPress: inputEnterKeyHandler }),
                        "x",
                        react_1.default.createElement(components_1.NumberInput, { min: 0, size: "s", view: "normal", value: height, onUpdate: setHeight, placeholder: (0, forms_1.i18n)('image_size_height'), className: b('input', { type: 'height' }), onKeyPress: inputEnterKeyHandler })) })),
            react_1.default.createElement(base_1.default.Footer, { onSubmit: handleSubmit }))));
};
exports.ImageForm = ImageForm;
