"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderSearchPopup = exports.SearchPopup = exports.SearchCard = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../../../../classname");
const search_1 = require("../../../../i18n/search");
const handlers_1 = require("../../../../utils/handlers");
const b = (0, classname_1.cn)('search-card');
const noop = () => { };
const SearchCard = ({ onChange = noop, onClose = noop, onSearchPrev = noop, onSearchNext = noop, onConfigChange = noop, }) => {
    const [query, setQuery] = (0, react_1.useState)('');
    const [isCaseSensitive, setIsCaseSensitive] = (0, react_1.useState)(false);
    const [isWholeWord, setIsWholeWord] = (0, react_1.useState)(false);
    const textInputRef = (0, react_1.useRef)(null);
    const setInputFocus = () => {
        var _a;
        (_a = textInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    };
    const handleInputChange = (event) => {
        const { target: { value }, } = event;
        setQuery(value);
        onChange(value);
    };
    const handleClose = () => {
        setQuery('');
        onClose(query);
        setInputFocus();
    };
    const handlePrev = () => {
        onSearchPrev(query);
        setInputFocus();
    };
    const handleNext = () => {
        onSearchNext(query);
        setInputFocus();
    };
    const handleIsCaseSensitive = () => {
        onConfigChange({
            isCaseSensitive: !isCaseSensitive,
            isWholeWord,
        });
        setIsCaseSensitive((isCaseSensitive) => !isCaseSensitive);
        setInputFocus();
    };
    const handleIsWholeWord = () => {
        onConfigChange({
            isCaseSensitive,
            isWholeWord: !isWholeWord,
        });
        setIsWholeWord((isWholeWord) => !isWholeWord);
        setInputFocus();
    };
    const handleSearchKeyPress = (0, handlers_1.enterKeyHandler)(handleNext);
    return (react_1.default.createElement(uikit_1.Card, { className: b() },
        react_1.default.createElement("div", { className: b('header') },
            react_1.default.createElement("span", { className: b('title') },
                " ",
                (0, search_1.i18n)('title')),
            react_1.default.createElement(uikit_1.Button, { onClick: handleClose, size: "s", view: "flat" },
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.Xmark, size: 14 }))),
        react_1.default.createElement(uikit_1.TextInput, { controlRef: textInputRef, className: (0, uikit_1.sp)({ mb: 2 }), size: "s", autoFocus: true, onKeyPress: handleSearchKeyPress, onChange: handleInputChange, value: query, endContent: react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(uikit_1.Button, { onClick: handlePrev },
                    react_1.default.createElement(uikit_1.Icon, { data: icons_1.ChevronUp, size: 12 })),
                react_1.default.createElement(uikit_1.Button, { onClick: handleNext },
                    react_1.default.createElement(uikit_1.Icon, { data: icons_1.ChevronDown, size: 12 }))) }),
        react_1.default.createElement(uikit_1.Checkbox, { size: "m", onUpdate: handleIsCaseSensitive, checked: isCaseSensitive, className: (0, uikit_1.sp)({ mr: 4 }) }, (0, search_1.i18n)('label_case-sensitive')),
        react_1.default.createElement(uikit_1.Checkbox, { size: "m", onUpdate: handleIsWholeWord, checked: isWholeWord }, (0, search_1.i18n)('label_whole-word'))));
};
exports.SearchCard = SearchCard;
const SearchPopup = (_a) => {
    var { open, anchor, onClose } = _a, props = tslib_1.__rest(_a, ["open", "anchor", "onClose"]);
    const anchorRef = (0, react_1.useRef)(anchor);
    return (react_1.default.createElement(uikit_1.Popup, { onEscapeKeyDown: onClose, open: anchorRef.current && open, anchorRef: anchorRef, placement: "bottom-end" },
        react_1.default.createElement(exports.SearchCard, Object.assign({ onClose: onClose }, props))));
};
exports.SearchPopup = SearchPopup;
exports.SearchPopup.displayName = 'SearchPopup';
function renderSearchPopup(_a) {
    var { anchor, open, onClose } = _a, props = tslib_1.__rest(_a, ["anchor", "open", "onClose"]);
    return (react_1.default.createElement(react_1.default.Fragment, null, anchor && react_1.default.createElement(exports.SearchPopup, Object.assign({ open: open, onClose: onClose, anchor: anchor }, props))));
}
exports.renderSearchPopup = renderSearchPopup;
