"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolbarButton = exports.ToolbarButtonView = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../classname");
const common_1 = require("../i18n/common");
const lodash_1 = require("../lodash");
const const_1 = require("./const");
const b = (0, classname_1.cn)('toolbar-button');
exports.ToolbarButtonView = react_1.default.forwardRef(function ToolbarButtonView({ icon, title, hint, hotkey, disabledPopoverVisible = true, hintWhenDisabled, active, enabled, onClick, className, }, ref) {
    var _a;
    const disabled = !active && !enabled;
    const titleText = (0, lodash_1.isFunction)(title) ? title() : title;
    const hintText = (0, lodash_1.isFunction)(hint) ? hint() : hint;
    const hideHintWhenDisabled = hintWhenDisabled === false || !disabledPopoverVisible || !disabled;
    const hintWhenDisabledText = typeof hintWhenDisabled === 'string'
        ? hintWhenDisabled
        : typeof hintWhenDisabled === 'function'
            ? hintWhenDisabled()
            : (0, common_1.i18n)('toolbar_action_disabled');
    return (react_1.default.createElement(uikit_1.Popover, { content: hintWhenDisabledText, disabled: hideHintWhenDisabled, tooltipContentClassName: b('action-disabled-tooltip'), placement: ['bottom'] },
        react_1.default.createElement(uikit_1.ActionTooltip, { openDelay: const_1.ToolbarTooltipDelay.Open, closeDelay: const_1.ToolbarTooltipDelay.Close, description: hintText, title: titleText, hotkey: hotkey },
            react_1.default.createElement(uikit_1.Button, { size: "m", ref: ref, selected: active, disabled: disabled, view: active ? 'normal' : 'flat', onClick: onClick, className: b(null, [className]), extraProps: { 'aria-label': titleText } },
                react_1.default.createElement(uikit_1.Icon, { data: icon.data, size: (_a = icon.size) !== null && _a !== void 0 ? _a : 16 })))));
});
function ToolbarButton(props) {
    const { id, editor, focus, isActive, isEnable, exec, onClick } = props;
    const active = isActive(editor);
    const enabled = isEnable(editor);
    return (react_1.default.createElement(exports.ToolbarButtonView, Object.assign({}, props, { active: active, enabled: enabled, onClick: () => {
            focus();
            exec(editor);
            onClick === null || onClick === void 0 ? void 0 : onClick(id);
        } })));
}
exports.ToolbarButton = ToolbarButton;
