import { __rest } from "tslib";
import React from 'react';
import { Icon } from '@gravity-ui/uikit';
import { cnIconRefuge } from './IconRefuge.classname';
import './IconRefuge.css';
/**
 * Creates a div wrapper above the icon, which sets the min-width and min-height
 * equal to its refugeSize prop, puts the original icon in the middle and
 * proxies the rest of the propses to it.
 *
 * The component is made in order to add margins to icons that have
 * margins are indented, but their svg exactly wrap around the edges of the image
 *
 * It seems that all icons will be square, if not, then you can simply add
 * refugeWidth and refugeHeight
 */
export const IconRefuge = (_a) => {
    var { refugeSize, containerClassName, containerStyle, widthOnly, inlineIcon, title, 'aria-label': ariaLabel } = _a, props = __rest(_a, ["refugeSize", "containerClassName", "containerStyle", "widthOnly", "inlineIcon", "title", 'aria-label']);
    if (props.size === refugeSize && !title && !ariaLabel && !containerStyle) {
        return React.createElement(Icon, Object.assign({}, props));
    }
    return (React.createElement("div", { className: cnIconRefuge({ inline: inlineIcon }, containerClassName), title: title, "aria-label": ariaLabel, style: Object.assign({ minWidth: refugeSize, minHeight: widthOnly ? undefined : refugeSize }, containerStyle) },
        React.createElement(Icon, Object.assign({ size: refugeSize }, props))));
};
IconRefuge.displayName = 'Icon';
