import React from 'react';
import { HelpPopover } from '@gravity-ui/components';
import { Hotkey, Icon, List, Popup } from '@gravity-ui/uikit';
import { cn } from '../../../classname';
import { i18n } from '../../../i18n/suggest';
import { isFunction } from '../../../lodash';
import { ErrorLoggerBoundary } from '../../../react-utils/ErrorBoundary';
import './component.css';
const b = cn('command-menu');
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
const ITEM_HEIGHT = 28; // px
const VISIBLE_ITEMS_COUNT = 10;
const MAX_LIST_HEIGHT = ITEM_HEIGHT * VISIBLE_ITEMS_COUNT; // px
function calcListHeight(itemsCount) {
    if (itemsCount <= 0)
        return undefined;
    return Math.min(MAX_LIST_HEIGHT, itemsCount * ITEM_HEIGHT);
}
export const CommandMenuComponent = ({ anchor, currentIndex, items, onClick, onEnterKeyDown, onEscapeKeyDown, onOutsideClick, }) => {
    if (!anchor)
        return null;
    return (React.createElement(Popup, { open: true, anchorRef: { current: anchor }, placement: placement, onEnterKeyDown: onEnterKeyDown, onEscapeKeyDown: onEscapeKeyDown, onOutsideClick: onOutsideClick },
        React.createElement("div", { className: b() },
            React.createElement(List, { virtualized: true, items: items, sortable: false, filterable: false, emptyPlaceholder: i18n('empty-msg'), itemHeight: ITEM_HEIGHT, itemsHeight: calcListHeight(items.length), renderItem: renderItem, deactivateOnLeave: false, activeItemIndex: currentIndex, onItemClick: (_item, index) => onClick(index), className: b('list'), itemClassName: b('list-item') }))));
};
function renderItem({ id, title, icon, hotkey, hint }) {
    const titleText = isFunction(title) ? title() : title;
    const hintText = isFunction(hint) ? hint() : hint;
    return (React.createElement("div", { key: id, className: b('item', { id }) },
        React.createElement(Icon, { data: icon.data, size: 20, className: b('item-icon') }),
        React.createElement("div", { className: b('item-body') },
            React.createElement("span", { className: b('item-title') }, titleText),
            React.createElement("div", { className: b('item-extra') },
                hotkey && React.createElement(Hotkey, { value: hotkey, className: b('item-hotkey') }),
                hintText && React.createElement(HelpPopover, { className: b('item- hint'), content: hintText })))));
}
export function render(props) {
    return (React.createElement(ErrorLoggerBoundary, null,
        React.createElement(CommandMenuComponent, Object.assign({}, props))));
}
