import { hasParentNodeOfType } from 'prosemirror-utils';
import { isFunction } from '../../../lodash';
import { textblockTypeInputRule } from '../../../utils/inputrules';
import { withLogAction } from '../../../utils/keymap';
import { CodeBlockHighlight } from './CodeBlockHighlight/CodeBlockHighlight';
import { CodeBlockSpecs } from './CodeBlockSpecs';
import { newlineInCode, resetCodeblock, setCodeBlockType } from './commands';
import { cbAction, codeBlockType } from './const';
import { codeBlockPastePlugin } from './plugins/codeBlockPastePlugin';
export { resetCodeblock } from './commands';
export { codeBlockNodeName, CodeBlockNodeAttr, codeBlockLangAttr, codeBlockType, } from './CodeBlockSpecs';
export const CodeBlock = (builder, opts) => {
    var _a;
    builder.use(CodeBlockSpecs, opts);
    builder.addKeymap((deps) => {
        const { codeBlockKey } = opts;
        const bindings = { Enter: newlineInCode, Backspace: resetCodeblock };
        if (codeBlockKey) {
            bindings[codeBlockKey] = withLogAction('code_block', setCodeBlockType(deps));
        }
        return bindings;
    });
    builder.addInputRules(({ schema }) => ({ rules: [codeBlockRule(codeBlockType(schema))] }));
    builder.addAction(cbAction, (deps) => {
        const cb = codeBlockType(deps.schema);
        return {
            isActive: (state) => hasParentNodeOfType(cb)(state.selection),
            isEnable: setCodeBlockType(deps),
            run: setCodeBlockType(deps),
        };
    });
    builder.addPlugin(codeBlockPastePlugin, builder.Priority.High);
    if (isFunction(opts.langs)) {
        builder.use(opts.langs);
    }
    else {
        builder.use(CodeBlockHighlight, (_a = opts.langs) !== null && _a !== void 0 ? _a : {});
    }
};
// Given a code block node type, returns an input rule that turns a
// textblock starting with three backticks into a code block.
function codeBlockRule(nodeType) {
    return textblockTypeInputRule(/^```$/, nodeType);
}
