import { logger } from '../../../../logger';
import { getProportionalSize } from '../../../../utils';
import { FilesBatchUploadProcess } from '../../../behavior/utils/upload';
import { imageType } from '../../../markdown';
import { IMG_MAX_HEIGHT } from '../const';
import { createImageNode, loadImage } from '../utils';
import { ImageSkeletonDescriptor } from './skeleton';
export class ImagesUploadProcess extends FilesBatchUploadProcess {
    constructor(view, files, uploadHandler, position, opts) {
        super(view, files, uploadHandler);
        this.initPosition = position;
        this.createImage = createImageNode(imageType(this.view.state.schema), opts);
        this.enableNewImageSizeCalculation = opts.enableNewImageSizeCalculation;
    }
    async createSkeleton() {
        return new ImageSkeletonDescriptor(this.initPosition, await getSkeletonSize(this.files, {
            enableNewImageSizeCalculation: this.enableNewImageSizeCalculation,
        }));
    }
    async createPMNode(res) {
        return this.createImage(res);
    }
}
async function getSkeletonSize(files, opts) {
    const skeletonSize = { width: '300', height: '200' };
    if (files.length === 1) {
        try {
            const size = await loadImage(files[0]).then((opts === null || opts === void 0 ? void 0 : opts.enableNewImageSizeCalculation) ? calcSkeletonSizeNew : calcSkeletonSize);
            skeletonSize.width = String(size.width);
            skeletonSize.height = String(size.height);
        }
        catch (err) {
            logger.error(err);
        }
    }
    return skeletonSize;
}
function calcSkeletonSize({ width, height }) {
    if (height <= IMG_MAX_HEIGHT)
        return { width, height };
    const ratio = IMG_MAX_HEIGHT / height; // ratio<1
    return { height: IMG_MAX_HEIGHT, width: width * ratio };
}
function calcSkeletonSizeNew({ width, height }) {
    return getProportionalSize({
        width,
        height,
        imgMaxHeight: IMG_MAX_HEIGHT,
    });
}
