import React, { useCallback, useEffect, useRef } from 'react';
import { cn } from '../../../../../classname';
import { useNodeEditing, useNodeHovered } from '../../../../../react-utils';
import { useNodeResizing } from '../../../../../react-utils/useNodeResizing';
import { removeNode } from '../../../../../utils';
import { Resizable } from '../../../../behavior/Resizable/Resizable';
import { ImgSizeAttr } from '../../ImgSizeSpecs';
import { imageRendererKey } from '../../const';
import { ImgSettingsButton } from './ImgSettingsButton';
import './ImgNodeView.css';
export const cnImgSizeNodeView = cn('img-size-node-view');
export const ImageNodeView = ({ node, view, getPos, updateAttributes, }) => {
    const imageContainerRef = useRef(null);
    const imageRef = useRef(null);
    const alt = node.attrs[ImgSizeAttr.Alt] || '';
    const initialHeight = node.attrs[ImgSizeAttr.Height];
    const initialWidth = node.attrs[ImgSizeAttr.Width];
    const src = node.attrs[ImgSizeAttr.Src] || '';
    const title = node.attrs[ImgSizeAttr.Title] || '';
    const isNodeHovered = useNodeHovered(imageContainerRef);
    const [edit, setEditing, unsetEdit, toggleEdit] = useNodeEditing({
        nodeRef: imageContainerRef,
        view,
    });
    const handleResize = useCallback(({ width, height }) => {
        updateAttributes({
            width: width === undefined ? undefined : String(Math.round(width)),
            height: height === undefined ? undefined : String(Math.round(height)),
            name: title,
            alt,
        });
    }, [alt, title, updateAttributes]);
    const { state, startResizing } = useNodeResizing({
        width: initialWidth,
        height: initialHeight,
        ref: imageRef,
        onResize: handleResize,
    });
    const style = {
        transition: 'width 0.15s ease-out, height 0.15s ease-out',
    };
    if (state.width) {
        style.width = `${state.width}px`;
        if (state.height) {
            style.aspectRatio = state.width / state.height;
            style.height = 'auto;';
        }
    }
    else if (state.height) {
        style.height = `${state.height}px`;
    }
    const handleDelete = useCallback(() => {
        const pos = getPos();
        if (pos === undefined)
            return;
        removeNode({
            node,
            pos,
            tr: view.state.tr,
            dispatch: view.dispatch,
        });
        view.focus();
    }, [getPos, node, view]);
    const createHandleResize = (direction) => (event) => {
        startResizing(event, direction);
    };
    useEffect(() => {
        var _a;
        if ((_a = imageRendererKey.getState(view.state)) === null || _a === void 0 ? void 0 : _a.linkAdded) {
            setEditing();
        }
    }, [view, setEditing]);
    return (React.createElement("div", { ref: imageContainerRef },
        React.createElement(Resizable, { hover: isNodeHovered, resizing: state.resizing, onResizeLeft: createHandleResize('left'), onResizeRight: createHandleResize('right') },
            React.createElement(ImgSettingsButton, { node: node, view: view, getPos: getPos, updateAttributes: updateAttributes, visible: isNodeHovered && !edit && !state.resizing, edit: edit, toggleEdit: toggleEdit, nodeRef: imageRef, onDelete: handleDelete, unsetEdit: unsetEdit }),
            React.createElement("img", { ref: imageRef, src: src, alt: alt, style: style }))));
};
