import React from 'react';
import { Tabs, TextInput } from '@gravity-ui/uikit';
import { cn } from '../classname';
import { i18n } from '../i18n/forms';
import { isFunction } from '../lodash';
import { enterKeyHandler } from '../utils/handlers';
import { TextInputFixed } from './TextInput';
import Form from './base';
import { ButtonAttach } from './components';
const b = cn('file-form');
export const FileForm = ({ className, autoFocus, onCancel, onSubmit, onAttach, loading, }) => {
    const [tabId, setTabId] = React.useState(() => isFunction(onAttach) ? "attach" /* TabId.Attach */ : "link" /* TabId.Link */);
    const [src, setSrc] = React.useState('');
    const [name, setName] = React.useState('');
    const shouldRenderTabs = isFunction(onAttach);
    React.useLayoutEffect(() => {
        if (!shouldRenderTabs && tabId === "attach" /* TabId.Attach */) {
            setTabId("link" /* TabId.Link */);
        }
    }, [shouldRenderTabs, tabId]);
    const handleSubmit = () => {
        onSubmit({
            src: src.trim(),
            name: name.trim(),
        });
    };
    const inputEnterKeyHandler = enterKeyHandler(handleSubmit);
    return (React.createElement(Form.Form, { className: b(null, [className]) },
        shouldRenderTabs && (React.createElement(Tabs, { activeTab: tabId, onSelectTab: setTabId, items: [
                { id: "attach" /* TabId.Attach */, title: i18n('common_tab_attach') },
                { id: "link" /* TabId.Link */, title: i18n('common_tab_link') },
            ] })),
        tabId === "attach" /* TabId.Attach */ && onAttach && (React.createElement(React.Fragment, null,
            React.createElement(Form.Layout, null, i18n('file_upload_help')),
            React.createElement(Form.Footer, { onCancel: onCancel },
                React.createElement(ButtonAttach, { multiple: true, buttonProps: {
                        size: 's',
                        view: 'action',
                        loading,
                    }, onUpdate: onAttach }, i18n('common_action_upload'))))),
        tabId === "link" /* TabId.Link */ && (React.createElement(React.Fragment, null,
            React.createElement(Form.Layout, null,
                React.createElement(Form.Row, { label: i18n('common_link'), help: i18n('file_link_help'), control: React.createElement(TextInputFixed, { size: "s", view: "normal", value: src, onUpdate: setSrc, placeholder: "https://", autoFocus: autoFocus, className: b('input', { type: 'link' }), onKeyPress: inputEnterKeyHandler }) }),
                React.createElement(Form.Row, { label: i18n('file_name'), control: React.createElement(TextInput, { size: "s", view: "normal", value: name, onUpdate: setName, className: b('input', { type: 'name' }), onKeyPress: inputEnterKeyHandler }) })),
            React.createElement(Form.Footer, { onCancel: onCancel, onSubmit: handleSubmit })))));
};
