import React from 'react';
import { TextInput } from '@gravity-ui/uikit';
import { i18n } from '../i18n/forms';
import { enterKeyHandler } from '../utils/handlers';
import { TextInputFixed } from './TextInput';
import { UrlInputRow } from './UrlInputRow';
import Form from './base';
export const LinkForm = React.memo(function LinkForm({ className, autoFocus, initialUrl, initialText, readOnlyText, onSubmit, onCancel, }) {
    const [url, setUrl] = React.useState(initialUrl !== null && initialUrl !== void 0 ? initialUrl : '');
    const [text, setText] = React.useState(initialText !== null && initialText !== void 0 ? initialText : '');
    const handleSubmit = () => {
        onSubmit({ url, text });
    };
    const inputEnterKeyHandler = enterKeyHandler(handleSubmit);
    return (React.createElement(Form.Form, { className: className },
        React.createElement(Form.Layout, null,
            React.createElement(Form.Row, { label: i18n('common_link'), help: i18n('link_url_help'), control: React.createElement(UrlInputRow, { href: url, input: React.createElement(TextInputFixed, { size: "s", view: "normal", value: url, onUpdate: setUrl, autoFocus: autoFocus, placeholder: "https://", onKeyPress: inputEnterKeyHandler }) }) }),
            React.createElement(Form.Row, { label: i18n('link_text'), help: i18n('link_text_help'), control: React.createElement(TextInput, { size: "s", view: "normal", value: text, onUpdate: setText, disabled: readOnlyText, onKeyPress: inputEnterKeyHandler }) })),
        React.createElement(Form.Footer, { onCancel: onCancel, onSubmit: handleSubmit })));
});
